@@ -4,7 +4,7 @@
  *
  * @brief mptcpd @c struct @c sockaddr related utility functions.
  *
- * Copyright (c) 2019-2021, Intel Corporation
+ * Copyright (c) 2019-2022, Intel Corporation
  */
 
 #include <string.h>
@@ -14,9 +14,9 @@
 #include <mptcpd/private/sockaddr.h>
 
 
-bool mptcpd_sockaddr_storage_init(struct in_addr  const *addr4,
+bool mptcpd_sockaddr_storage_init(in_addr_t const *addr4,
                                   struct in6_addr const *addr6,
-                                  unsigned short port,
+                                  in_port_t port,
                                   struct sockaddr_storage *addr)
 {
         if ((addr4 == NULL && addr6 == NULL) || addr == NULL)
@@ -27,7 +27,7 @@ bool mptcpd_sockaddr_storage_init(struct in_addr  const *addr4,
 
                 a->sin_family      = AF_INET;
                 a->sin_port        = port;
-                a->sin_addr.s_addr = addr4->s_addr;
+                a->sin_addr.s_addr = *addr4;
         } else {
                 struct sockaddr_in6 *const a =
                         (struct sockaddr_in6 *) addr;
