@@ -115,8 +115,9 @@ static bool append_addr_attr(struct l_genl_msg *msg,
                 struct sockaddr_in const *const addr4 =
                         (struct sockaddr_in *) addr;
 
-                data = &addr4->sin_addr;
-                len  = sizeof(addr4->sin_addr);
+                // IPv4 address in network byte order.
+                data = &addr4->sin_addr.s_addr;
+                len  = sizeof(addr4->sin_addr.s_addr);
         } else {
                 if (local)
                         type = MPTCP_ATTR_SADDR6;
@@ -162,7 +163,7 @@ static int mptcp_org_add_addr(struct mptcpd_pm *pm,
               Local address ID
               Local address family
               Local address
-              Local port (optional)
+              Local port (host byte order) (optional)
          */
 
         // Types chosen to match MPTCP genl API.
@@ -282,11 +283,11 @@ static int mptcp_org_add_subflow(struct mptcpd_pm *pm,
               Local address ID
               Remote address ID
               Remote address
-              Remote port
+              Remote port (host byte order)
 
               Optional attributes:
                   Local address
-                  Local port
+                  Local port (host byte order)
                   Backup priority flag
                   Network interface index
          */
@@ -381,9 +382,9 @@ static int mptcp_org_set_backup(struct mptcpd_pm *pm,
               Token
               Address family
               Local address
-              Local port
+              Local port (host byte order)
               Remote address
-              Remote port
+              Remote port (host byte order)
               Backup priority flag
          */
 
@@ -465,9 +466,9 @@ static int mptcp_org_remove_subflow(struct mptcpd_pm *pm,
               Token
               Address family
               Local address
-              Local port
+              Local port (host byte order)
               Remote address
-              Remote port
+              Remote port (host byte order)
          */
 
         uint16_t const family      = mptcpd_get_addr_family(local_addr);
